% function[U,t]=rough_road_input(N,H)
% Generates N points in time U(t) at a time interval of H with zero mean.
% N must be a power of 2! The N points will be generated
% so as to have a desired one-sided PSD defined in the function
% road_psd_smooth_highway(f). f is the frequency in Hertz. The
% frequency resolution will be 1/NH. The smallest frequency will be 0
% Hz and the largest frequency will be 1/2H Hz. Note, the frequency
% 1/2H is referred to as the folding frequency.

function[U,t]=rough_road_input(N,H)
no2=N/2;
% Step 1: Generate frequencies and PSD values at each frequency
f=0:1/(N*H):1/(2*H);
%Note, the first frequency is zero but we don't
%compute GY at f=0 to avoid potential computational problems. GY at f=0
%should be zero for a zero mean process.
GU(1)=0;
% If the desired PSD is defined in road_psd_smooth_highway, then use the next 3 lines.
for m1=2:no2+1
% USE NEXT LINE IF SIMULTAION IS FOR SMOOTH RUNWAY
%GU(m1)=road_psd_smooth_runway(f(m1));
% USE NEXT LINE IF SIMULTAION IS FOR ROUGH RUNWAY
%GU(m1)=road_psd_rough_runway(f(m1));
% USE NEXT LINE IF SIMULTAION IS FOR SMOOTH HIGHWAY
GU(m1)=road_psd_smooth_highway(f(m1));
% USE NEXT LINE IF SIMULTAION IS FOR HIGHWAY WITH GRAVEL
%GU(m1)=road_psd_gravel_highway(f(m1));
% USE NEXT LINE IF SIMULTAION IS FOR PASTURE IN FIELD
%GU(m1)=road_psd_pasture(f(m1));
end
% Step 1: Generate N/2 random phase angles with uniform denisty
% between 0 and 2pi
TH=random('unif',0,2*pi,no2,1);
% Step 2: Generate the appropriate amplitude at each frequency using
% the random phase angles.
for m=2:no2
C=sqrt(GU(m)*N*H/2);
CTH=cos(TH(m-1))*C;
STH=sin(TH(m-1))*C;
U(m)=CTH+1j*STH;
k=N+2-m;
U(k)=CTH-1j*STH;
end
% Step 3: Make sure the N/2+1 value is real that the negative frequency values
% will be the mirror image of the positive frequency values.
no2p1=no2+1;

C=sqrt(GU(no2p1)*N*H/2);
CTH=cos(TH(no2))*C;
U(no2p1)=CTH+1j*0.e0;
% Make sure the zero frequency value is zero to achieve a zero mean time series.
U(1)=0.e0+1j*0.e0;
% Generate Y(t), the inverse transform of the N Fourier Transform values Y(f).
[U]=ifft(U);
% Rescale to get the correct time series corresponding to this desired PSD.
for m=1:N
U(m)=real(U(m)/H);
end
% Plot the PSD and time series
%PLOT OF ROAD ELEVATIONS VS TIME
t=0:H:(N-1)*H;
t=t';
U=U';
figure
plot(t,U);
xlabel('Time (sec)')
ylabel('Output Variable Corresponding to Desired PSD')
grid on
%PLOT OF ROAD ELEVATIONS VS DISTANCE
%figure
%dist=30.*t;
%plot(dist,U);
%xlabel('Distance (m)')
%ylabel('Output Variable Corresponding to Desired PSD')
grid on
figure
plot((f/30),GU,'r','LineWidth',2);
xlabel('Spatial Frequency (cycles/m)');
ylabel('Desired PSD (m^2/cycles/m)');
grid on
% AXIS COMMAND FOR SMOOTH RUNWAY
%axis([0 0.02 0 4])
% AXIS COMMAND FOR ROUGH RUNWAY
axis([0 0.02 0 2]);
% AXIS COMMAND FOR SMOOTH HIGHWAY
%axis([0 0.2 0 3e-4])
% AXIS COMMAND FOR HIGHWAY WITH GRAVEL
%axis([0 0.02 0 1.2])
% AXIS COMMAND FOR PASTURE
%axis([0 0.05 0 1.5])

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
end
